#include "stdafx.h"
#include "MyProgress.h"
#include "resource.h"

// ********************************************************************************************

CMyProgress::CMyProgress(HWND xhWndProgress)
{
	hWndParent = GetParent(xhWndProgress);
	RECT rectChild,rectChildInt;
	RECT rectParent,rectParentInt;

	GetWindowRect(xhWndProgress,&rectChild);
	GetClientRect(xhWndProgress,&rectChildInt);
	GetWindowRect(hWndParent,&rectParent);
	GetClientRect(hWndParent,&rectParentInt);
	deltaX = rectChildInt.right-rectChildInt.left;
	deltaY = rectChildInt.bottom-rectChildInt.top;
	int decalX,decalY;
	decalX = (rectParent.right-rectParent.left)-(rectParentInt.right-rectParentInt.left);
	decalY = (rectParent.bottom-rectParent.top)-(rectParentInt.bottom-rectParentInt.top);
	posX = rectChild.left-rectParent.left-decalX+2;
	posY = rectChild.top-rectParent.top-decalY+2;

	hInst = (HINSTANCE) GetWindowLong(xhWndProgress,GWL_HINSTANCE);
	DestroyWindow (xhWndProgress);

	hFont = CreateFont (17,6,0,0,0,0,0,0,0,0,0,0,0,"Tahoma");
	percent = 0;
	rgbBackGround = RGB(192,192,192);
	rgbBorderBackGround = RGB(128,128,128);
	rgbProgress = RGB(192,0,0);
	rgbBorderProgress = RGB(128,0,0);
	rgbText = RGB (0,0,128);
	xJustify = jus_center;
	yJustify = jus_center;

	unknownProgress = FALSE;
	text = "";

	CreateControl(LoadCursor (NULL, IDC_ARROW));
}

// ********************************************************************************************

CMyProgress::~CMyProgress()
{
	DestroyWindow(hWndParent);
}

// ********************************************************************************************

LRESULT CALLBACK ProcMyProgress (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	 HDC         hDC;
	 PAINTSTRUCT ps ;

	 static CMyProgress *progress;

     switch (message)
     {
		case WM_INITDIALOG:
			progress = (CMyProgress *) lParam; // pour dialog
			return TRUE;

		case WM_DESTROY: 
			return TRUE;

		case WM_PAINT:
			hDC = BeginPaint (hwnd, &ps);
			
			HBRUSH hBrushOld;
			HBRUSH hBrushBackGround = CreateSolidBrush(progress->rgbBackGround);
			HBRUSH hBrushProgress = CreateSolidBrush(progress->rgbProgress);

			HPEN hPenOld;
			HPEN hPenBackGroundBorder = CreatePen(PS_SOLID,1,progress->rgbBorderBackGround);
			HPEN hPenProgressBorder = CreatePen(PS_SOLID,1,progress->rgbBorderProgress);
			
			HFONT hFontOld;

			hBrushOld = (HBRUSH) SelectObject(hDC,hBrushBackGround);
			hPenOld= (HPEN)SelectObject(hDC,hPenBackGroundBorder);
			hFontOld = (HFONT) SelectObject(hDC,progress->hFont);
	
			// Affichage du fond
			Rectangle(hDC,0,0,progress->deltaX,progress->deltaY);
			
			// Affichage de la progression

			SelectObject(hDC,hBrushProgress);
			SelectObject(hDC,hPenProgressBorder);

			if (progress->percent>0.0) 
				Rectangle(hDC,2,2,(int)((progress->deltaX-4)*progress->percent)+2,progress->deltaY-2);

			// Affichage  de la progression inconnue
			if (progress->unknownProgress)
			{
				// determination du sens de fondue de couleur
				// indiceUnknown 0 a 255 ==> 0 a 255  <=\
				// indiceUnknown 256 a 511 ==> 255 a 0   |
				// indiceUnknown 512 a 639 ==> 0 a 255  /  etc ...
				int indice;
				if (! (progress->indiceUnknown&0x100)) indice = progress->indiceUnknown&0xff;
				else indice = 0x100 - progress->indiceUnknown&0xff;

				DeleteObject(hBrushBackGround);
				COLORREF rgbBackGround = RGB(0,indice,0);
				HBRUSH hBrushProgress = CreateSolidBrush(rgbBackGround);

				SelectObject(hDC,hBrushProgress);
				SelectObject(hDC,hPenProgressBorder);

				Rectangle(hDC,
				(int)((progress->deltaX-4)*progress->percent)+2,
				2,
				(int)((progress->deltaX-4)*(progress->percent+progress->percentUnknown))+2,
				progress->deltaY-2);
			}
			
			// affichage du text
			SetBkMode(hDC ,TRANSPARENT);
			SetTextColor(hDC, progress->rgbText);

			RECT rect;
			rect.left = 0;
			rect.top = 0;
			rect.right = progress->deltaX;
			rect.bottom = progress->deltaY;
			DrawText(hDC,progress->text,progress->text.size,&rect,DT_CENTER|DT_VCENTER|DT_SINGLELINE);

			SelectObject(hDC,hBrushOld);
			SelectObject(hDC,hPenOld);
			SelectObject(hDC,hFontOld);

			DeleteObject(hBrushBackGround);
			DeleteObject(hBrushProgress);
			DeleteObject(hPenBackGroundBorder);
			DeleteObject(hPenProgressBorder);

			EndPaint (hwnd, &ps);
			return TRUE;
     }
	 return FALSE;
} 

// ********************************************************************************************

BOOL CMyProgress::CreateControl(HCURSOR hCursor)
{
	// creation de la nouvelle bar de progression
	hWndProgress = CreateDialogParam(hInst,MAKEINTRESOURCE(IDD_MY_PROGRESS),hWndParent,(DLGPROC)ProcMyProgress,(LPARAM)this);

	SetWindowPos(hWndProgress,NULL,posX,posY,deltaX,deltaY ,0);
    ShowWindow(hWndProgress ,SW_SHOW);

	return TRUE;
}

// ********************************************************************************************

void CMyProgress::Set(float xpercent, char *xtext)
{
	unknownProgress = FALSE;
	text = xtext;
	percent = xpercent;

	InvalidateRect(hWndProgress,NULL,FALSE);
}

// ********************************************************************************************

void CMyProgress::SetUnknown(float xpercentUnknown,char *xtext)
{
	if (!unknownProgress) indiceUnknown = -1;
	unknownProgress = TRUE;
	percentUnknown = xpercentUnknown;
	indiceUnknown ++;
	text = xtext;
	InvalidateRect(hWndProgress,NULL,FALSE);
}
