#ifndef INCLUDE_ANALYSE_H
#define INCLUDE_ANALYSE_H

#include "resource.h"
// *************************************************************************************
#define uchar unsigned char

class CElem
{
	public:
		uchar *buffer;
		long size;

	public:
		CElem()
		{
			buffer = NULL;
			size = 0;
		}

		CElem(uchar *_buffer,long _size)
		{
			Set(_buffer,_size);
		}

		void Set(uchar *_buffer,long _size)
		{
			buffer = _buffer;
			size = _size;
		}

		void ExtractText(CMyString &result,long delta=0,long debIndex=0)
		{
			if (delta==0) delta=size;
			delta = min(delta,size-debIndex);
			for (long i=0;i<delta;i++) 
				if (isprint(buffer[i+debIndex])) result<<(char)buffer[i+debIndex];
				else result<<'?'; 
		}

		void ExtractCode(CMyString &result)
		{
			for (long i=0;i<size;i++) result<<(int)buffer[i]<<' ';
		}

		void Extract(CMyString &result)
		{
			result<<"\"";
			ExtractText(result);
			result<<"\" {";		
			ExtractCode(result);
			result<<" }";
		}


		uchar operator[](long index)
		{
			if (index<0) return buffer[size+index];
			return buffer[index];
		}

		uchar * Get(long index)
		{
			if (index<0) return buffer+size+index;
			return buffer+index;
		}
		
		BOOL operator==(CElem &elem)
		{
			if (size!=elem.size) return FALSE;
			return !memcmp(buffer,elem.buffer,size);
		}

		BOOL IsSubWord (CElem &elem)
		{
			if (operator==(elem)) return FALSE;
			long maxTest=min(size,elem.size);
			long delta=labs(size-elem.size);

 			for (long offset=0;offset<=delta;offset++)
			{
				BOOL isValide = TRUE;
				for (long len=0;len<maxTest;len++)
				{
					if (operator[](len+offset)!=elem[len]) 
					{ 
						isValide = FALSE; 
						break; 
					}
				}
				if (isValide) return TRUE;
			}
			return FALSE;
		}
					
		BOOL SameSuffix (CElem &elem)
		{
			if (size==elem.size) return FALSE;
			long maxTest=min(size,elem.size);
			for (long len=1;len<maxTest+1;len++)
			{
				if (operator[](-len)!=elem[-len]) return FALSE;
			}
			return TRUE;
		}
};

// *************************************************************************************

class CBuffer
{
	public:
		CElem buffer1;
		CElem buffer2;

	public:
		CBuffer(uchar *ptrBuffer1,long size1,uchar *ptrBuffer2,long size2)
		{
			buffer1.Set(ptrBuffer1,size1);
			buffer2.Set(ptrBuffer2,size2);
		}

		long PGDC(long i,long j)
		{
			long len;
			for (len=0;len<min(buffer1.size-i,buffer2.size-j);len++)
			{
				if (buffer1[i+len] != buffer2[j+len]) break;
			}
			return len;
		}
};

// *************************************************************************************

class CLstBuffer
{
	public:
		CMyList<long> lstBuffer;

		BOOL operator+=(long value)
		{
			for (lstBuffer=0;lstBuffer.More();lstBuffer++)
			{
				long *elem = lstBuffer.GetElemPtr();
				if (*elem == value) return FALSE;
			}
			lstBuffer+=value;
			return TRUE;
		}

		void Extract(CMyString &temp,BOOL bracket = FALSE)
		{
			if (bracket) temp<<"{";
			for (lstBuffer=0;lstBuffer.More();lstBuffer++)
			{
				temp<<" "<<lstBuffer.GetElem();
			}
			if (bracket) temp<<" }";
		}

		BOOL Content(long index,long sizeContener,long sizeElem)
		{
			for (lstBuffer=0;lstBuffer.More();lstBuffer++)
			{
				int debIndex = lstBuffer.GetElem();
				if ( (index>=debIndex) && ((index+sizeElem)<=(debIndex+sizeContener))) return TRUE;
			}
			return FALSE;
		}
};

// *************************************************************************************

typedef struct
{
	CLstBuffer lstBuffer1;
	CLstBuffer lstBuffer2;
	CElem elem;
	long nbSame;
} TListModele;

// *************************************************************************************
#define PERSO_END_ANALYSE 1024
DWORD WINAPI ChildThreadProcedure (IN LPVOID paramVoid);

class CLstModele
{
	public:
		HWND m_hWndParent;
		uchar *m_buffer1;
		uchar *m_buffer2; 
		CMyString m_file1,m_file2;
		int m_size;
		CMyProgress *m_progress;
		BOOL m_endFind;

	public:
		CMyList<TListModele> list;

	public:
		CLstModele (HWND hWndParent)
		{
			m_hWndParent = hWndParent;
		}

		void Extract (CMyString &result)
		{
			for (list=0;list.More();list++)
			{
				TListModele modele = list.GetElem();
				modele.elem.Extract(result);
				result<<"\n   In1 ";
				modele.lstBuffer1.Extract(result);
				result<<"\n   In2 ";
				modele.lstBuffer2.Extract(result);
				result<<"\n";
			}
		}

		void ArrangeOrder()
		{
			TListModele *tl;
			CMyList<long> lstBuffer;

			for (list=0;list.More();list++)
			{
				tl=list.GetElemPtr();
				lstBuffer.SuprAll();

				tl->nbSame = 0;
				for (tl->lstBuffer1.lstBuffer=0;tl->lstBuffer1.lstBuffer.More();tl->lstBuffer1.lstBuffer++)
				{
					long index1 = tl->lstBuffer1.lstBuffer.GetElem();

					for (tl->lstBuffer2.lstBuffer=0;tl->lstBuffer2.lstBuffer.More();tl->lstBuffer2.lstBuffer++)
					{
						long index2 = tl->lstBuffer2.lstBuffer.GetElem();
						if (index1==index2)
						{
							lstBuffer+=index1;
							tl->nbSame++;
							tl->lstBuffer1.lstBuffer.SuprCurrent();
							tl->lstBuffer2.lstBuffer.SuprCurrent();
						}
					}
				}
				for (lstBuffer.GoLast();lstBuffer.More();lstBuffer--)
				{
					tl->lstBuffer1.lstBuffer.AddFirst(lstBuffer.GetElem());
					tl->lstBuffer2.lstBuffer.AddFirst(lstBuffer.GetElem());
				}
			}
		}

		BOOL Analyse(char *file1,char *file2,long size,CMyProgress *progress,BOOL endFind)
		{
			m_file1=file1;
			m_file2=file2;
			m_size=size;
			m_progress=progress;
			m_endFind=endFind;
			DWORD uUnusedThreadId;
			HANDLE hChildThread = (HANDLE) CreateThread ( 
				NULL, 
				0, 
				ChildThreadProcedure, 
				(void*) this, 
				0, 
				&uUnusedThreadId);
			return (BOOL)hChildThread ;
		}
};


DWORD WINAPI ChildThreadProcedure (IN LPVOID paramVoid)
{
	CLstModele *modele = (CLstModele *) paramVoid;

	modele->list.SuprAll();


	FILE *fich1 = fopen(modele->m_file1,"rb");
	FILE *fich2 = fopen(modele->m_file2,"rb");
	if (!fich1) MessageBox(NULL,modele->m_file1,"Can't Read this file",0); return FALSE;
	if (!fich2) MessageBox(NULL,modele->m_file2,"Can't Read this file",0); return FALSE;


	modele->m_buffer1 = new uchar[modele->m_size];
	modele->m_buffer2 = new uchar[modele->m_size];
	if (!modele->m_buffer1 || !modele->m_buffer2 || !fich1 ||!fich2) 
		return FALSE;
	
	int size1;
	int size2;
	if (modele->m_endFind)
	{
		fseek(fich1,-modele->m_size,SEEK_END);
		fseek(fich2,-modele->m_size,SEEK_END);
	}
	size1 = fread(modele->m_buffer1,1,modele->m_size,fich1);
	size2 = fread(modele->m_buffer2,1,modele->m_size,fich2);

	fclose(fich1);
	fclose(fich2);

	CMyString temp;

	// MAJ des taille des fichier lu
	temp<<"(Read "<<size1<<" bytes)";
	Static_SetText(GetDlgItem(modele->m_hWndParent,IDC_STATIC_FILE1),temp);
	temp = "";
	temp<<"(Read "<<size2<<" bytes)";
	Static_SetText(GetDlgItem(modele->m_hWndParent,IDC_STATIC_FILE2),temp);

	int i,j;
	int count = -1;

	CBuffer buffer(modele->m_buffer1,size1,modele->m_buffer2,size2);
	TListModele *ptrModele;
	CElem elem;
	int len;
	long oldPercent;
	float percent=-1;

	for (i=0;i<size1;i++)
	{
		for (j=0;j<size2;j++)
		{
			count++;
			percent = (float)count/(float)(size1*size2);
			if (oldPercent<(long)(percent*100))
			{	
				oldPercent = (long)(percent*100);
				temp="";
				temp<<(long)(percent*100)<<"%";
				modele->m_progress->Set(percent,temp);
			}
			len = buffer.PGDC(i,j);
			if (len)
			{
				elem.Set(buffer.buffer1.Get(i),len);
				BOOL isFinish = FALSE;
				BOOL isAdd = FALSE;

				for (modele->list=0;modele->list.More() && !isFinish;modele->list++)
				{
					ptrModele = modele->list.GetElemPtr();
					if (ptrModele->elem.IsSubWord(elem))
					{
						if (ptrModele->lstBuffer2.Content(j,ptrModele->elem.size,elem.size)) 
						{
							isAdd = TRUE;
							isFinish = TRUE;
						}
					}
				}
				
				for (modele->list=0;modele->list.More() && !isFinish;modele->list++)
				{
					ptrModele = modele->list.GetElemPtr();
					if (ptrModele->elem==elem)
					{
						ptrModele->lstBuffer1 += i; 
						ptrModele->lstBuffer2 += j; 
						isFinish = TRUE;
						isAdd = TRUE;
					}
				}

				if (!isAdd)
				{
					TListModele modeleCur;
					modeleCur.elem = elem;
					modeleCur.lstBuffer1+=i;
					modeleCur.lstBuffer2+=j;
					modele->list+=modeleCur;
				}
			}
		}
	}
//	delete buffer1;
//	delete buffer2;
	temp="";
	temp<<"Global view: ("<<modele->list.GetNbElem()<<" modeles)";
	Static_SetText(GetDlgItem(modele->m_hWndParent,IDC_STATIC_GLOBAL),temp);

	modele->ArrangeOrder();

	SendMessage(modele->m_hWndParent,WM_COMMAND,PERSO_END_ANALYSE,0);
	return TRUE;
}



#endif